package Controlador;

import Modelo.Periodico;
import Modelo.UsoBD;
import Vista.Final_Results;
import Vista.Summary;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.table.DefaultTableModel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;

/**
 * Clase controlador de la vista Final_Results.
 * @author Adrián
 */
public class control_Final_Results implements ActionListener{

    private UsoBD mod; 
    private Periodico period;
    private Final_Results miVista;
    /**
     * Método constructor.
     * Se añaden los listener de los objetos de la vista para que muestren los eventos del usuario en la interfaz.
     * @param mod Instancia para interactuar con la base de datos.
     * @param period Instancia para obtener la infromación de los Periódicos.
     * @param miVista Vista Final_Results correspondiente.
     */
    public control_Final_Results(UsoBD mod,Periodico period,Final_Results miVista){
        this.mod=mod;
        this.period=period;
        this.miVista=miVista;
        this.miVista.Titulares.addActionListener(this);
        this.miVista.Palabras.addActionListener(this);
        this.miVista.Caracteres.addActionListener(this);
        this.miVista.Volver.addActionListener(this);
        this.miVista.ABC.addActionListener(this);
        this.miVista.El_País.addActionListener(this);
        this.miVista.La_Vanguardia.addActionListener(this);
    }
    /**
     * Método llamado al mostrar la vista para mostar los gráficos y tablas.
     */
    public void iniciar() {
        miVista.setTitle("Results");
        miVista.setSize(1100, 700);
        miVista.setLocationRelativeTo(null);
        graficar();
    }
    /**
     * Método para la creación y actualización de los gráficos, con los datos alamcenados en la base de datos.
     */
    public void graficar(){
        //Se determina que opción está seleccionada por el usuario.
        String elección_fuente="";
        if(miVista.ABC.isSelected()){
            elección_fuente=miVista.ABC.getText();
        }
        if(miVista.El_País.isSelected()){
            elección_fuente=miVista.El_País.getText();
        }
        if(miVista.La_Vanguardia.isSelected()){
            elección_fuente=miVista.La_Vanguardia.getText();
        }
        //Se obtienen los modelos para mostrar en la tabla, realizando la correspondiente consulta a la base de datos.
        DefaultTableModel Titulares;
        DefaultTableModel Palabras;
        DefaultTableModel Caracteres;
        Titulares = mod.agruparTitulares(elección_fuente);
        Palabras = mod.agruparPalabras(elección_fuente);
        Caracteres = mod.agruparCaracteres(elección_fuente);
        //Dependiendo de qué opción haya seleccionado, se muestra una tabla u otra.
        if(miVista.Titulares.isSelected()){
            miVista.tblConsultas.setModel(Titulares);
        }
        if(miVista.Caracteres.isSelected()){
            miVista.tblConsultas.setModel(Caracteres);
        }
        if(miVista.Palabras.isSelected()){
            miVista.tblConsultas.setModel(Palabras);
        }
        //Creación del gráfico de tarta, con la proporción del número de titulares.
        DefaultPieDataset num_titulares = new DefaultPieDataset();
        for(int i = 0; i < Titulares.getRowCount(); i++){
            num_titulares.setValue(Titulares.getValueAt(i, 0).toString(), 
                    Integer.parseInt(Titulares.getValueAt(i, 1).toString()));
        }
        //Creación del gráfico de barras, con la media de palabras.
        DefaultCategoryDataset media_palabras = new DefaultCategoryDataset();
        for(int i = 0; i < Palabras.getRowCount(); i++){
            media_palabras.setValue(Double.parseDouble(Palabras.getValueAt(i, 1).toString()), Palabras.getValueAt(i, 0).toString(), Palabras.getValueAt(i, 0).toString()); 
        }
        //Creación del gráfico de barras, con la media de caracteres.
        DefaultCategoryDataset media_caracteres = new DefaultCategoryDataset();          
        for(int i = 0; i < Caracteres.getRowCount(); i++){
            media_caracteres.setValue(Double.parseDouble(Caracteres.getValueAt(i, 1).toString()), Caracteres.getValueAt(i, 0).toString(), Caracteres.getValueAt(i, 0).toString());
        }
        //Se establecen los gráficos en los paneles correspondientes de la vista.
        JFreeChart ch1 = ChartFactory.createPieChart("Titulares Totales", num_titulares,true, true, false);
        ChartPanel cp1 = new ChartPanel(ch1);
        cp1.getChart().removeLegend();
        miVista.panel4.setLayout(new java.awt.BorderLayout());
        miVista.panel4.add(cp1,BorderLayout.CENTER);
        miVista.panel4.validate();
        JFreeChart ch2 = ChartFactory.createBarChart("Media Palabras", "Fuente", "Palabras", media_palabras,PlotOrientation.HORIZONTAL, true, true, false);
        ChartPanel cp2 = new ChartPanel(ch2);
        cp2.getChart().removeLegend();
        miVista.panel5.setLayout(new java.awt.BorderLayout());
        miVista.panel5.add(cp2,BorderLayout.CENTER);
        miVista.panel5.validate();
        JFreeChart ch3 = ChartFactory.createBarChart(" Media Caracteres", "Fuente", "Caracteres", media_caracteres,PlotOrientation.HORIZONTAL, true, true, false);
        ChartPanel cp3 = new ChartPanel(ch3);
        cp3.getChart().removeLegend();
        miVista.panel6.setLayout(new java.awt.BorderLayout());
        miVista.panel6.add(cp3,BorderLayout.CENTER);
        miVista.panel6.validate();
    }
    /**
     * Dependiendo del evento en la vista Final_Results se mostrará una información u otra.
     * @param e 
     */
    public void actionPerformed(ActionEvent e) {
        //En la tabla se muestra el número de titulares por sección del periódico 
        //seleccionado.
        if (e.getSource() == miVista.Titulares) {
            graficar();
        }
        //En la tabla se muestra la media de palabras por sección del periódico 
        //seleccionado.
        if (e.getSource() == miVista.Palabras) {
            graficar();
        }
        //En la tabla se muestra la media de caracteres por sección del periódico 
        //seleccionado.
        if (e.getSource() == miVista.Caracteres) {
            graficar();
        }
        //Se muestra la tabla y gráficos correspondientes con el ABC.
        if (e.getSource() == miVista.ABC) {
            graficar();
        }
        //Se muestra la tabla y gráficos correspondientes con el El País.
        if (e.getSource() == miVista.El_País) {
            graficar(); 
        }
        //Se muestra la tabla y gráficos correspondientes con La Vanguardia.
        if (e.getSource() == miVista.La_Vanguardia) {
            graficar(); 
        }
        //Se vuelve a la vista anterior de Summary.
        if (e.getSource() == miVista.Volver) {
            finalizar();
        }
    }
    /**
     * Cuando el usuario decide vover a la vista de Summary.
     */
    public void finalizar() {
        miVista.dispose();
        Summary frm = new Summary();
        control_Summary ctrl = new control_Summary(period,mod,frm);
        mod.setHora_inicio();
        ctrl.iniciar();
        frm.setVisible(true);
    }
}