package Controlador;

import Modelo.Periodico;
import Modelo.UsoBD;
import Vista.Results;
import Vista.Summary;
import com.jcraft.jsch.JSchException;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;

/**
 * Clase controlador de la vista Results.
 * @author Adrián
 */
public class control_Results implements ActionListener , WindowListener{
    private UsoBD mod; 
    private Periodico period;
    public Results miVista;
    private int contador=0;
    /**
     *Constructor de la clase.
     * Se añaden los listener de los objetos de la vista para que muestren los eventos del usuario en la interfaz.
     * @param mod Instancia para interactuar con la base de datos.
     * @param period Instancia para obtener la infromación de los Periódicos.
     * @param miVista Vista Results correspondiente.
     */
    public control_Results(UsoBD mod,Periodico period,Results miVista){
        this.mod=mod;
        this.period=period;
        this.miVista=miVista;
        this.miVista.Titulares.addActionListener(this);
        this.miVista.Palabras.addActionListener(this);
        this.miVista.Caracteres.addActionListener(this);
        this.miVista.ABC.addActionListener(this);
        this.miVista.El_País.addActionListener(this);
        this.miVista.La_Vanguardia.addActionListener(this);
        this.miVista.addWindowListener(this);
    }
    /**
     * Método que comienza la actualización de la vista, con los nuevos datos.
     */
    public void iniciar() {
        contador++;
        miVista.setTitle("Extracción de titulares");
        miVista.setLocationRelativeTo(null);
        graficar();
        barra();
    }
    /**
     * Cambia el valor de la barra de progreso, actualizándola.
     */
    public void barra(){
        //Si la elección es el nº de titulares,hace la proporción de los que
        //se han almacenado con los que hay que almacenar.
        if(mod.getElección().equals("Titulares")){
            try {
                int max = mod.getMax_titulares();
                int saved = mod.Guardados_Desde_Ahora();
                if(miVista.jProgressBar1.getValue()<100){
                    float div = (float)saved/max;
                    div=100*div;
                    miVista.jProgressBar1.setValue(Math.round(div));
                    act();
                }
            } catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
                Logger.getLogger(control_Results.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        //Si la elección es el número de horas, hace la proporción entre el tiempo  que ha pasado
        //desde que se inició la herramienta, con la hora prevista para la finzalización.
        if(mod.getElección().equals("Horas")){
            Date ini= mod.getInicio();
            Date fin = mod.getHora_fin();
            Calendar cal = Calendar.getInstance();
            cal.setTime(new Date());	
            Date mitad = cal.getTime();
            int diferencia1=(int) ((fin.getTime()-ini.getTime())/1000);
            int diferencia2=(int) ((fin.getTime()-mitad.getTime())/1000);
            if(miVista.jProgressBar1.getValue()<100){
                float div = (float)diferencia2/diferencia1;
                div=100*(1-div);                  
                miVista.jProgressBar1.setValue(Math.round(div));
                act();
            }
        }
    }
    //Método de actualización de los campos correspondientes del último titular almacenado.
    public void act(){
        int ult;
        String [] ult_added=new String[3];
        try {
            ult = mod.numTitular();
            ult_added=mod.busca_ult();
            if(ult >0){            
                miVista.Ultimo.setText(ult_added[0]);
                miVista.Fuente.setText(ult_added[1]);
                miVista.Tipo.setText(ult_added[2]);
            }
            } catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
                Logger.getLogger(control_Results.class.getName()).log(Level.SEVERE, null, ex);
            }
    }
    /**
     * Método para la creación y actualización de los gráficos, con los datos alamcenados en la base de datos.
     */
    public void graficar(){
        //Se determina que opción está seleccionada por el usuario.
        String elección_fuente="";
        if(miVista.ABC.isSelected()){
            elección_fuente=miVista.ABC.getText();
        }
        if(miVista.El_País.isSelected()){
            elección_fuente=miVista.El_País.getText();
        }
        if(miVista.La_Vanguardia.isSelected()){
            elección_fuente=miVista.La_Vanguardia.getText();
        }
        //Se obtienen los modelos para mostrar en la tabla, realizando la correspondiente consulta a la base de datos.
        DefaultTableModel Titulares;
        DefaultTableModel Palabras;
        DefaultTableModel Caracteres;
        Titulares = mod.agruparTitulares(elección_fuente);
        Palabras = mod.agruparPalabras(elección_fuente);
        Caracteres = mod.agruparCaracteres(elección_fuente);
        SwingUtilities.invokeLater(
            new Runnable(){
                public void run(){
                    //Dependiendo de qué opción haya seleccionado, se muestra una tabla u otra.
                    if(miVista.Titulares.isSelected()){
                        miVista.tblConsultas.setModel(Titulares);
                        }
                    if(miVista.Caracteres.isSelected()){
                        miVista.tblConsultas.setModel(Caracteres);
                    }
                    if(miVista.Palabras.isSelected()){
                        miVista.tblConsultas.setModel(Palabras);
                    }
                }
            });
        //Creación del gráfico de tarta, con la proporción del número de titulares.
        DefaultPieDataset num_titulares = new DefaultPieDataset();
        for(int i = 0; i < Titulares.getRowCount(); i++){
            num_titulares.setValue(Titulares.getValueAt(i, 0).toString(),
                    Integer.parseInt(Titulares.getValueAt(i, 1).toString()));
        }
        //Creación del gráfico de barras, con la media de palabras.
        DefaultCategoryDataset media_palabras = new DefaultCategoryDataset();
        for(int i = 0; i < Palabras.getRowCount(); i++){
            media_palabras.setValue(Double.parseDouble(Palabras.getValueAt(i, 1).toString()), 
                    Palabras.getValueAt(i, 0).toString(), Palabras.getValueAt(i, 0).toString()); 
        }
        //Creación del gráfico de barras, con la media de caracteres.
        DefaultCategoryDataset media_caracteres = new DefaultCategoryDataset();
        for(int i = 0; i < Caracteres.getRowCount(); i++){
            media_caracteres.setValue(Double.parseDouble(Caracteres.getValueAt(i, 1).toString()), 
                    Caracteres.getValueAt(i, 0).toString(), Caracteres.getValueAt(i, 0).toString());
        }
        //Se establecen los gráficos en los paneles correspondientes de la vista.
        JFreeChart ch1 = ChartFactory.createPieChart("Titulares Totales", num_titulares,true, true, false);
        ChartPanel cp1 = new ChartPanel(ch1);
        cp1.getChart().removeLegend();
        JFreeChart ch2 = ChartFactory.createBarChart("Media Palabras", "Fuente", "Palabras", media_palabras,PlotOrientation.HORIZONTAL, true, true, false);
        ChartPanel cp2 = new ChartPanel(ch2);
        cp2.getChart().removeLegend();
        JFreeChart ch3 = ChartFactory.createBarChart(" Media Caracteres", "Fuente", "Caracteres", media_caracteres,PlotOrientation.HORIZONTAL, true, true, false);
        ChartPanel cp3 = new ChartPanel(ch3);
        cp3.getChart().removeLegend();
        if(contador>1){
            miVista.panel4.remove(cp1);
            miVista.panel5.remove(cp2);
            miVista.panel6.remove(cp3);
        }
        miVista.panel4.setLayout(new java.awt.BorderLayout());
        miVista.panel4.add(cp1,BorderLayout.CENTER);
        miVista.panel4.validate();
        miVista.panel5.setLayout(new java.awt.BorderLayout());
        miVista.panel5.add(cp2,BorderLayout.CENTER);
        miVista.panel5.validate();
        miVista.panel6.setLayout(new java.awt.BorderLayout());
        miVista.panel6.add(cp3,BorderLayout.CENTER);
        miVista.panel6.validate();
        cp1=null;
        cp2=null;
        cp3=null;
        ch1=null;
        ch2=null;
        ch3=null;
    }
    /**
     * Dependiendo del evento en la vista Results se mostrará una información u otra.
     * @param e 
     */
    public void actionPerformed(ActionEvent e) {
        //En la tabla se muestra el número de titulares por sección del periódico 
        //seleccionado y se actualiza la barra de progreso.
        if (e.getSource() == miVista.Titulares) {
            graficar();
            barra();
        }
        //En la tabla se muestra la media de palabras por sección del periódico
        //seleccionado y se actualiza la barra de progreso.
        if (e.getSource() == miVista.Palabras) {
            graficar();
            barra();   
        }
        //En la tabla se muestra la media de caracteres por sección del periódico
        //seleccionado y se actualiza la barra de progreso.
        if (e.getSource() == miVista.Caracteres) {
            graficar();
            barra();   
        }
        //Se muestra la tabla y gráficos correspondientes con el ABC y se actualiza la barra de progreso.
        if (e.getSource() == miVista.ABC) {
            graficar();
            barra();   
        }
        //Se muestra la tabla y gráficos correspondientes con El País y se actualiza la barra de progreso.
        if (e.getSource() == miVista.El_País) {
            graficar();
            barra();             
        }
        //Se muestra la tabla y gráficos correspondientes con La Vanguardia y se actualiza la barra de progreso.
        if (e.getSource() == miVista.La_Vanguardia) {
            graficar();
            barra();             
        }
    }
    /**
     * Cuando ha finalizado el proceso de extracción, se pasa a la vista de Summary.
     */
    public void finalizar() {
        miVista.dispose();
        Summary frm = new Summary();
        control_Summary ctrl = new control_Summary(period,mod,frm);
        ctrl.iniciar();
        frm.setVisible(true);
        frm.setResizable(false);
    }
    /**
     * Impide que el usuario pueda interactuar con la interfaz, mientras se está actualizando la base de datos.
     * Desactiva los elementos con los que el usuario puede interactuar.
     */
    public void desactivar(){
        miVista.Actualizando.setText("Actualizando base de datos");
        miVista.ABC.setEnabled(false);
        miVista.El_País.setEnabled(false);
        miVista.La_Vanguardia.setEnabled(false);
        miVista.Caracteres.setEnabled(false);
        miVista.Palabras.setEnabled(false);
        miVista.Titulares.setEnabled(false);
    }
    /**
     * Paermite al usuario interactuar con la interfaz, mientras no se está actualizando la base de datos.
     */
    public void activar(){
        miVista.Actualizando.setText("Consulte gráficos");
        miVista.ABC.setEnabled(true);
        miVista.El_País.setEnabled(true);
        miVista.La_Vanguardia.setEnabled(true);
        miVista.Caracteres.setEnabled(true);
        miVista.Palabras.setEnabled(true);
        miVista.Titulares.setEnabled(true);
    }

    @Override
    public void windowOpened(WindowEvent e) {
    }
    
    /**
     * Método para capturar el evento de cerrar la ventana, antes de cerrar la aplicación.
     * Se muestra un mensaje al usuario de si realmente está seguro de acabar con la extraacción de titulares.
     * @param e 
     */
    @Override
    public void windowClosing(WindowEvent e) {
        miVista.setDefaultCloseOperation(miVista.DO_NOTHING_ON_CLOSE);
        if (JOptionPane.showConfirmDialog(miVista, "El proceso de extracción finalizará. ¿Realmente quiere salir?",
                "¿Salir?", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION){
            System.exit(0);
        } 
    }

    @Override
    public void windowClosed(WindowEvent e) {   
    }
    
    @Override
    public void windowIconified(WindowEvent e) {
    }

    @Override
    public void windowDeiconified(WindowEvent e) {
    }

    @Override
    public void windowActivated(WindowEvent e) {
    }

    @Override
    public void windowDeactivated(WindowEvent e) {
    }
}