package Controlador;

import Levenshtein.LevenshteinDistancia;
import Modelo.Periodico;
import Modelo.UsoBD;
import Vista.Final_Results;
import Vista.Summary;
import Vista.Titulares;
import com.jcraft.jsch.JSchException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;

/**
 * Clase controlador de la vista Summary.
 * @author Adrián
 */
public class control_Summary implements ActionListener{
    private UsoBD mod;
    public Summary miVista;
    public Periodico period;
    /**
     * Método constructor.
     * Se añaden los listener de los objetos de la vista para que muestren los eventos del usuario en la interfaz.
     * @param period Instancia para obtener la infromación de los Periódicos.
     * @param mod Instancia para interactuar con la base de datos.
     * @param miVista Vista Summary correspondiente.
     */
    public control_Summary(Periodico period,UsoBD mod ,Summary miVista){
        this.mod=mod;
        this.period=period;
        this.miVista=miVista;
        this.miVista.Graficos.addActionListener(this);
        this.miVista.Procesamiento.addActionListener(this);
        this.miVista.Importar_Negativas.addActionListener(this);
        this.miVista.Importar_Positivas.addActionListener(this);
        this.miVista.Importar_Busqueda.addActionListener(this);
        this.miVista.Titulares.addActionListener(this);
    }
    /**
     * Método llamado al mostrar la vista.
     * Realiza una consulta a la base de datos apra mostrar el número de titulares almacenados, por cada periódico.
     */
    public void iniciar() {
        try {
            int guardados1 =mod.busca_num("El Pais");
            int guardados2 = mod.busca_num("ABC");
            int guardados3= mod.busca_num("La Vanguardia");
            miVista.Total.setText(Integer.toString(mod.numTitular()));
            miVista.El_País.setText(Integer.toString(guardados1));
            miVista.ABC.setText(Integer.toString(guardados2));
            miVista.La_Vanguardia.setText(Integer.toString(guardados3));
        } catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
            Logger.getLogger(control_Summary.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    /**
     * Captura de los eventos del usuario en la Vista.
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        //Se muestra la vista con los gráficos.
        if (e.getSource() == miVista.Graficos) {
            finalizar();
        }
        //Se realiza el procesamiento de palabras.
        if (e.getSource() == miVista.Procesamiento) {
            procesar();
            JOptionPane.showMessageDialog(null,"Procesamiento finalizado");
        }
        //Se importa el archivo CSV con las palabras negativas.
        //Se realiza la interacción correspondiente con la base de datos y se muestra un mensaje al finalizar.
        if (e.getSource() == miVista.Importar_Negativas) {
            mod.borrarNegativas();
            mod.importarcsvNegativas();
            JOptionPane.showMessageDialog(null,"La palabras negativas han sido importadas");
        }
        //Se importa el archivo CSV con las palabras positivas.
        //Se realiza la interacción correspondiente con la base de datos y se muestra un mensaje al finalizar.
        if (e.getSource() == miVista.Importar_Positivas) {
            mod.borrarPositivas();
            mod.importarcsvPositivas();
            JOptionPane.showMessageDialog(null,"La palabras positivas han sido importadas");
        }
        //Se importa el archivo CSV con las palabras de búsqueda.
        //Se realiza la interacción correspondiente con la base de datos y se muestra un mensaje al finalizar.
        if (e.getSource() == miVista.Importar_Busqueda) {
            mod.borrarBusqueda();
            mod.importarcsvBusqueda();
            JOptionPane.showMessageDialog(null,"Las palabras para buscar han sido importadas");
        }
        //Se muestra la vista con los titulares.
        if (e.getSource() == miVista.Titulares) {
            titulares();
        }    
    }
    /**
     * Método para ir a la vista de Final_Results a ver los gráficos.
     */
    public void finalizar() {
        miVista.dispose();
        Final_Results frm = new Final_Results();
        control_Final_Results ctrl = new control_Final_Results(mod,period,frm);
        ctrl.iniciar();
        frm.setVisible(true);
        frm.setResizable(false);
    }
    /**
     * Método para ir a la vista de Final_Results a ver los gráficos.
     */
    public void titulares() {
        miVista.dispose();
        Titulares frm = new Titulares();
        control_Titulares ctrl = new control_Titulares(mod,period,frm);
        ctrl.iniciar();
        frm.setVisible(true);
        frm.setResizable(false);
    }
    /**
     * Método para realizar el procesamiento de las palabras.
     */
    public void procesar(){
        try {
            //Lo primero es poner los campos de positiva, negativa y busqueda a false.
            mod.resetNegativas();
            mod.resetPositivas();
            mod.resetBusqueda();
            //Obtención de las palabras consideradas negativas.
            ArrayList <String> negativas = mod.getNegativas();
            //Obtención de las palabras consideradas positivas.
            ArrayList <String> positivas = mod.getPositivas();
            //Obtención de las palabras de búsqueda.
            ArrayList <String> busqueda = mod.getBusqueda();
            //Obtención de las palabras almacenadas en la base de datos.
            ArrayList <String> palabras = mod.buscaPalabras();
            //Creación de Arrays de String con las palabras para poder iterar.
            String[] lista_negativas = new String[negativas.size()];
            String[] lista_positivas = new String[positivas.size()];
            String[] lista_busqueda = new String[busqueda.size()];
            String[] lista_palabras = new String[palabras.size()];
            for (int i=0;i<negativas.size();i++){
                lista_negativas[i]=negativas.get(i);
            }
            for (int i=0;i<positivas.size();i++){
                lista_positivas[i]=positivas.get(i);
            }
            for (int i=0;i<busqueda.size();i++){
                lista_busqueda[i]=busqueda.get(i);
            }
            for (int i=0;i<palabras.size();i++){
                lista_palabras[i]=palabras.get(i);
            }
            //Para cada palabra distinta de las demas alamcenada en la base de datos.
            for(int i=0;i<lista_palabras.length;i++){
                String aux=lista_palabras[i];//Palabra de la base de datos.
                //Pequeñas excepciones.
                if(aux.length()>0 && !aux.equals("valencia") && !aux.equals("renuncia")
                       && !aux.equals("renuncian") && !aux.equals("renuncias")&&
                       !aux.equals("occidente")&&!aux.equals("tentado")){
                    
                    String str1 = aux;
                    String str2;
                    int contador1=0;
                    int contador3=0;
                    LevenshteinDistancia ld = new LevenshteinDistancia();
                    //Para cada una de las palabras negativas, se calcula la similaridad con la palabra guardada en aux.
                    for (int j=0;j<lista_negativas.length;j++){
                        str2=lista_negativas[j];
                        ld.setWords(str1, str2);
                        //Si hay más de un 75% de afinidad se considera negativa.
                        if(ld.getAfinidad() * 100>=75){
                            contador1++;
                        }
                    }        
                    if(contador1>0){
                        mod.setNegativa(aux);
                    }
                    //Para cada una de las palabras positivas, se calcula la similaridad con la palabra guardada en aux.
                    for (int h=0;h<lista_positivas.length;h++){
                        str2=lista_positivas[h];
                        ld.setWords(str1, str2);
                        //Si hay más de un 75% de afinidad se considera positiva.
                        if(ld.getAfinidad() * 100>=75){
                            contador3++;
                        }
                    }        
                    if(contador3>0){
                        mod.setPositiva(aux);
                    }
                    //Para cada una de las palabras de búsqueda, se calcula la similaridad con la palabra guardada en aux.
                    for (int k=0;k<lista_busqueda.length;k++){
                        str2=lista_busqueda[k];    
                        ld.setWords(str1, str2);
                        //Si hay más de un 75% de afinidad se considera de búsqueda.
                        if(ld.getAfinidad() * 100>=75){
                            mod.setBusqueda(aux);
                        }
                    }
                }
            }
        } catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
            Logger.getLogger(control_Summary.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
