package Modelo;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * Clase que guarda la información propia de los titulares.
 * @author Adrián
 */
public class FeedMessage {
    String title;
    String titulo_limpio;
    String campo_fecha;
    Calendar fecha;
    int palabras;
    int caracteres;
    String nombre_periodico;
    String tipo_informacion;
    int num;

    //Getter paara obtener la fecha.
    public Calendar getFecha() {
        return fecha;
    }
    //Setter para establecer la fecha.
    public void setFecha( Calendar fecha) {
        this.fecha = fecha;
    }
    //Gettter para obtener el campo de la fecha.
    public String getCampo_fecha() {
        return campo_fecha;
    }
    //Setter para establecer el campo de la fecha.
    public void setCampo_fecha(String campo_fecha) {    
        this.campo_fecha = campo_fecha;
        dividirfecha(campo_fecha);
    }
    //Getter para obtener el identificador del titular.
    public int getNum() {    
        return num;
    }
    //Setter para establecer el identificador del titular.
    public void setNum(int num) {
        this.num = num;
    }
    //Getter para obtener el titular completo.
    public String getTitle() {
        return title;
    }
    //Setter para establecer el titular.
    public void setTitle(String title) {
        title= quitar_signos(title);//Se quitan los signos
        String[] partes_titulo = title.split(" ");//Se obtienen las palabras del titular.
        estadisticas(title,partes_titulo);//Obtención de estadísticas del titular.
        String limpio=limpiar(title,partes_titulo);//Obtención del titular limpio.      
        limpio=limpio.toLowerCase();
        this.titulo_limpio=limpio;
        this.title = title;
    }
    //Getter para obtner el titular limpio.
    public String getTituloLimpio() {
        return titulo_limpio;
    }
    //Setter para establecer el titular limpio.
    public void setTituloLimpio(String titulo_limpio){
        this.titulo_limpio=titulo_limpio;
    }
    //Getter para obtener el número de palabras de un titular.
    public int getPalabras() {
        return palabras;
    }
    //Setter para establecer el número de palabras de un titular.
    public void setPalabras(int palabras) {
        this.palabras = palabras;
    }
    //Getter para obtener el número de caracteres de un titular.
    public int getCaracteres() {
        return caracteres;
    }
    //Setter para establecer el número de caracteres de un titular.
    public void setCaracteres(int caracteres) {
        this.caracteres = caracteres;
    }
    //Getter para obtener el nombre del periódico del titular.
    public String getNombre_periodico() {
        return nombre_periodico;
    }
    //Setter para establecer el nombre del periódico del titular.
    public void setNombre_periodico(String nombre_periodico) {
        this.nombre_periodico = nombre_periodico;
    }
    //Getter para obtner el tipo de información del titular.
    public String getTipo_informacion() {
        return tipo_informacion;
    }
    //Setter para establecer el tipo de información del titular.
    public void setTipo_informacion(String tipo_informacion) {
        this.tipo_informacion = tipo_informacion;
    }
    //Ca´culo de estadísticas básicas como son el número de caracteres y de palabras del titular.
    public void estadisticas(String title, String [] partes_titulo){
        setCaracteres(title.length());
        setPalabras(partes_titulo.length); 
    }
    //Método donde se reaaliza el proceso de limpieza del titular.
    //Se establece varias listas de palabras a eliminar.
    //Existe una lista de preposiciones, artículos, conjunciones y otras.
    public String limpiar(String title,String [] partes_titulo){
        String[] preposiciones = { "a", "ante", "bajo", "cabe", "con", "contra", "de", "desde", "en", "entre", "hacia",
            "hasta", "para", "por", "según", "sin", "so", "sobre", "tras","A", "Ante", "Bajo", "Cabe", "Con", "Contra", "De",
            "Desde", "En", "Entre", "Hacia", "Hasta", "Para", "Por", "Según", "Sin", "So", "Sobre", "Tras"};
        String[] articulos = { "el", "la", "los", "las", "un", "una", "unos", "unas","El", "La", "Los", "Las", "Un", "Una", "Unos", "Unas"};
        String[] conjunciones ={"y","e","ni","que","Y","Ni","Que","E","o","O","u","U","Qué","qué"};
        String[] otras ={"al","Al","del","Del","se","Se","te","Te","su","Su","mi","mis","sus","su","tus","Mi","Mis",
            "Sus","Su","Tus", "lo","Lo","este","Este","esta","Esta","ese","Ese","esa","Esa","eso","Eso","ha","has",
            "hemos","habeis","han","hubiese","hubiesen","hubiera","hubieran","hubiesemos","hubieramos","hubieseis",
            "hubierais","he","hubiesen","hubieran","Ha","Has","Hemos","Habeis","Han","Hubiese","Hubiesen","Hubiera",
            "Hubieran","Hubiesemos","Hubieramos","Hubieseis","Hubierais","He","Hubiesen","Hubieran","como","cómo",
            "Como","Cómo","le","Le","me","Me","Si","si","No","no","Son","son","pero","Pero","Más","más","Tú","Tu","tu","tú",
            "Blog","Blogs","blog","blogs"};
        String limpio="";
        //Para cada palabra del título se comprueba se si encuentra en alguna de las listas anteriores
        for(int i=0;i<partes_titulo.length;i++){
            String aux=partes_titulo[i];
            Boolean preposicion= false;
            Boolean articulo=false;
            Boolean conjuncion=false;
            Boolean otra=false;
            for(int j=0;j<preposiciones.length;j++){
                if(aux.equals(preposiciones[j])){
                    preposicion=true;
                }
            }
            for(int j=0;j<articulos.length;j++){
                if(aux.equals(articulos[j])){
                    articulo=true;
                }
            }
            for(int j=0;j<conjunciones.length;j++){
                if(aux.equals(conjunciones[j])){
                    conjuncion=true;
                }
            }
            for(int j=0;j<otras.length;j++){
                if(aux.equals(otras[j])){
                    otra=true;
                }
            }
            //Si está en la lista, la palabra se elimina y no se cuenta para determinar el titular limpio.
            if(preposicion==false && articulo==false &&conjuncion==false &&otra==false){
                if(limpio==null){
                    limpio=limpio+aux;
                }
                else{
                    limpio= limpio+" "+aux;
                }   
            }
        }
        return limpio;
    }
    //El campo obtenido del archivo XML se divide para obtener más campos de datos.
    public void dividirfecha(String campo_fecha){
        String[] partes_fecha = campo_fecha.split(" ");
        String[] partes_hora = partes_fecha[4].split(":");
        int mes=0;
        switch (partes_fecha[2]) {
            case "Jan":  mes=1; break;
            case "Feb":  mes=2; break;
            case "Mar":  mes=3; break;
            case "Apr":  mes=4; break;
            case "May":  mes=5; break;
            case "Jun":  mes=6; break;
            case "Jul":  mes=7; break;
            case "Aug":  mes=8; break;
            case "Sep":  mes=9; break;
            case "Oct": mes=10; break;
            case "Nov": mes=11; break;
            case "Dec": mes=12; break;
        }
        this.fecha= new GregorianCalendar(Integer.parseInt(partes_fecha[3]),
                                            mes-1,
                                            Integer.parseInt(partes_fecha[1]),
                                            Integer.parseInt(partes_hora[0]),
                                            Integer.parseInt(partes_hora[1]),
                                            Integer.parseInt(partes_hora[2]));
    
    }
    //Cambio de formato de la fecha.
    public String fechaString(){
        Date date = fecha.getTime();
        DateFormat dateFormat = new SimpleDateFormat("YYYY-MM-dd HH:mm:ss");  
        String strDate = dateFormat.format(date);
        return strDate;
    }
    //Eliminación de los signos.
    public String quitar_signos(String title){
        String sin_signos=title.replaceAll("[\"--+*/¿?.,;¡!:|‘’“”'«»]","");
        return sin_signos;
    }
}