package Modelo;

import com.jcraft.jsch.JSchException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import javax.swing.table.DefaultTableModel;

/**
 * Clase para interactuar con la base de datos.
 * @author Adrián
 */
public class UsoBD extends Conexion{
    int max_titulares=0;
    int guardados_inicio=0;
    String elección="";
    int valor=0;
    String hora_inicio;
    Date hora_fin;
    Date inicio;
    int ult;
    int num_negativas;
    int num_positivas;
    int num_busqueda;
    /**
     * 
     * @return Devuele la fecha de inicio de ejecución de la aplicación.
     */
    public Date getInicio() {
        return inicio;
    }
    /**
     * Para establecer la fecha de inicio de ejecución de la aplicación.
     * @param inicio 
     */
    public void setInicio(Date inicio) {
        this.inicio = inicio;
    }
    /**
     * 
     * @return Devuelve el número de titulares almacenados al inicio de la ejecución de la aplicación.
     */
    public int getGuardados_inicio() {
        return guardados_inicio;
    }
    /**
     * 
     * @return Obtiene el valor del campo límite especificado por el usuario.
     */
    public int getValor() {
        return valor;
    }
    /**
     * Dependiendo de la elección, establece el número de titulares a almacenar o la hora de finalización de la ejecución.
     * @param valor 
     */
    public void setValor(int valor) {
        this.valor = valor;
        if(elección.equals("Titulares")){
            setMax_titulares(valor);
        }
        if(elección.equals("Horas")){
            setHora_fin();
        }
    }
    /**
     * 
     * @return Obtiene la elección de finalización especificada por el usuario. 
     */
    public String getElección() {
        return elección;
    }
    /**
     * Establece la elección de finalización especificada por el usuario. 
     * @param elección 
     */
    public void setElección(String elección) {
        this.elección = elección;
    }
    /**
     * 
     * @return Devuelve la hora de inicio de la ejecución de la aplicación. 
     */
    public String getHora_inicio() {
        return hora_inicio;
    }
    /**
     * Establece la hora de inicio de la ejecución de la aplicación.
     */
    public void setHora_inicio() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());	
        Date tempDate = cal.getTime();
        DateFormat dateFormat = new SimpleDateFormat("YYYY-MM-dd HH:mm:ss");  
        String strDate = dateFormat.format(tempDate);
        this.hora_inicio = strDate;
    }
    /**
     * 
     * @return Devuelve la hora de finalización establecida para la ejecución de la aplicación.
     */
    public Date getHora_fin() {
        return hora_fin;
    }
    /**
     * Establece la hora de finalización establecida para la ejecución de la aplicación.
     */
    public void setHora_fin() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());	
        Date tempDate = cal.getTime();
        this.inicio=tempDate;
        cal.set(Calendar.MINUTE, cal.get(Calendar.MINUTE)+ (valor*60));
        tempDate = cal.getTime();
        this.hora_fin =tempDate;
    }
    /**
     * 
     * @return Obtiene el número máximo de titulares para almacenar.
     */
    public int getMax_titulares() {
        return max_titulares;
    }
    /**
     * Establece el número máximo de titulares para almacenar.
     * @param max_titulares 
     */
    public void setMax_titulares(int max_titulares){
        this.max_titulares = max_titulares;
    }
    /**
     * 
     * @return Obtiene el valor numérico del ultimo titular almacenado. Coincide con el núemro de titulares almacenados.
     */
    public int getUlt() {
        return ult;
    }
    /**
     * Establece elel valor numérico del ultimo titular almacenado.
     * @param ult 
     */
    public void setUlt(int ult) {
        this.ult = ult;
    }
    /**
     * 
     * @return Obtiene el número de palabras negativas.
     */
    public int getNum_negativas() {
        return num_negativas;
    }
    /**
     * Establece el número de palabras negativas.
     * @param num_negativas 
     */
    public void setNum_negativas(int num_negativas) {
        this.num_negativas = num_negativas;
    }
    /**
     * Getter del número de palabras positivas.
     * @return 
     */
    public int getNum_positivas() {
        return num_positivas;
    }
    /**
     * Setter del número de palabras positivas.
     * @param num_positivas 
     */
    public void setNum_positivas(int num_positivas) {
        this.num_positivas = num_positivas;
    }
    /**
     * Getter del número de palabras de búsqueda.
     * @return 
     */
    public int getNum_busqueda() {
        return num_busqueda;
    }
    /**
     * Setter del número de palabras de búsqueda.
     * @param num_busqueda 
     */
    public void setNum_busqueda(int num_busqueda) {
        this.num_busqueda = num_busqueda;
    }
    /**
     * Método para la inserción de titulares en la base de datos con los valores proporcionados.
     * @param tit
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */   
    public void createTit (FeedMessage tit) throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql = "INSERT INTO Titulares (titulo,titulo_limpio,fecha,palabras,caracteres,nombre_periodico,tipo_informacion,num) VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ps.setString(1,tit.getTitle());
        ps.setString(2,tit.getTituloLimpio());
        ps.setString(3,tit.fechaString());
        ps.setInt(4,tit.getPalabras());
        ps.setInt(5,tit.getCaracteres());
        ps.setString(6,tit.getNombre_periodico());
        ps.setString(7,tit.getTipo_informacion());
        ps.setInt(8,tit.getNum());
        ps.execute();
        ps.close();
        desconectar();
    }
    /**
     * Consulta  a la base de datos que devuelve el núemero de titulares almacenados a partir del inicio de la ejecución.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public int Guardados_Desde_Ahora () throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql =          " SELECT COUNT(*) AS total"
                            + " FROM Titulares"
                            + " WHERE fecha>'"+hora_inicio+"';";
  
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        int total=0;
        while (rs.next()) {
            total = rs.getInt("total");
        }
        ps.close();
        desconectar();
        return total;
    }
    /**
     * Consulta a la base de datos que devuelve el número de titulares almacenados.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public int numTitular () throws SQLException, JSchException, InstantiationException, IllegalAccessException { 
        String sql =          " SELECT COUNT(*) AS total"
                            + " FROM Titulares;";
                          
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        int total=0;
        while (rs.next()) {
            total = rs.getInt("total");
        }
        ps.close();
        setUlt(total);
        desconectar();
        return total;
    }
    /**
     * Consulta a la base de datos que devuelve el núemro de titulares de un periódico en concreto proporcioanado.
     * @param per
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public int busca_num(String per) throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql =          " SELECT COUNT(*) as total"
                            + " FROM Titulares"
                            + " WHERE nombre_periodico='"+per+"';";
                        
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        int total=0;
        while (rs.next()) {
        total = rs.getInt("total");
        }
        ps.close();
        desconectar();
        return total;
    }
    /**
     * Consulta a la base de datos que devuelve el número de palabras negativas almacenadas en su correspondiente tabla.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public int numNegativas () throws SQLException, JSchException, InstantiationException, IllegalAccessException { 
        String sql =          " SELECT COUNT(*) AS total"
                            + " FROM Negativas;";
                          
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        int total=0;
        while (rs.next()) {
        total = rs.getInt("total");
        }
        ps.close();
        setNum_negativas(total);
        desconectar();
        return total;
    }
    /**
     * Consulta a la base de datos que devuelve el número de palabras positivas almacenadas en su correspondiente tabla.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public int numPositivas () throws SQLException, JSchException, InstantiationException, IllegalAccessException {    
        String sql =          " SELECT COUNT(*) AS total"
                            + " FROM Positivas;";
                          
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        int total=0;
        while (rs.next()) {
        total = rs.getInt("total");
        }
        ps.close();
        setNum_positivas(total);
        desconectar();
        return total;
    }
    /**
     * Consulta a la base de datos que devuelve el número de palabrasde búsqueda almacenadas en su correspondiente tabla.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public int numBusqueda () throws SQLException, JSchException, InstantiationException, IllegalAccessException { 
        String sql =          " SELECT COUNT(*) AS total"
                            + " FROM Busqueda;";
                          
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        int total=0;
        while (rs.next()) {
            total = rs.getInt("total");
        }
        ps.close();
        setNum_busqueda(total);
        desconectar();
        return total;
    }
    /**
     * Consulta a la base de datos que devuelve el ultimo titular almacenado.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public String[] busca_ult () throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql =          " SELECT *"
                            + " FROM Titulares"
                            + " WHERE num="+getUlt()+";";
                         
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        String[] titulo=new String[3];
        while (rs.next()) {
            for(int i=0; i<3;i++){
            titulo[0] = rs.getString("titulo");
            titulo[1] = rs.getString("nombre_periodico");
            titulo[2] = rs.getString("tipo_informacion");
            }
        }
        ps.close();
        desconectar();
        return titulo;
    }
    /**
     * Consulta a la base de datos que busca si un titular está ya almacenado en la base de datos o no.
     * @param titu
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public Boolean buscaTitularespecif (String titu) throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql =          " SELECT *"
                            + " FROM Titulares"
                            + " WHERE titulo='"+titu+"';";
                            
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        
        if (rs.next() == false) {
            ps.close();
            desconectar();
            return true;
        }
        else{
            ps.close();
            desconectar();
            return false;
        }    
    }
    /**
     * Consulta a la base de datos que busca si una palabra de un mismo titular ya se encuentra almacenada en la base de datos.
     * @param palabra
     * @param nombre_periodico
     * @param tipo_informacion
     * @param num
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public Boolean buscaPalabraespecif (String palabra,String nombre_periodico, String tipo_informacion,int num) throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql =          " SELECT *"
                            + " FROM Palabras"
                            + " WHERE palabra='"+palabra+"' AND nombre_periodico='"+nombre_periodico+"' AND tipo_informacion ='"+tipo_informacion+"' AND num ='"+num+"';";
                          
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        if (rs.next() == false) {
            ps.close();
            desconectar();
            return true;
        }
        else{
            ps.close();
            desconectar();
            return false;
        }  
    }
    /**
     * Consulta a la base de datos que devuelve el modelo de la tabla con el número de titulares agrupados por sección.
     * @param fuente
     * @return 
     */
    public DefaultTableModel agruparTitulares(String fuente){
        DefaultTableModel miModelo = null;
        try{
            String titulos []= {"Fuentes|Tipo","Nº Titulares"};
            String dts [] = new String[2];
            miModelo = new DefaultTableModel(null, titulos);
            String sql = "select nombre_periodico,tipo_informacion, count(*) from Titulares where nombre_periodico='"+fuente+"' group by tipo_informacion;";
            PreparedStatement pst = hazConexion().prepareStatement(sql);
            ResultSet rs = pst.executeQuery();
            while(rs.next()){
               String juntar=rs.getString("nombre_periodico")+" | "+rs.getString("tipo_informacion");
               dts[0] = juntar;
               dts[1] = rs.getString("count(*)");
               miModelo.addRow(dts);
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
        desconectar();
        return miModelo;
    }
    /**
     * Consulta a la base de datos que devuelve el modelo de la tabla con la media de palabras agrupados por sección.
     * @param fuente
     * @return 
     */
    public DefaultTableModel agruparPalabras(String fuente){
        DefaultTableModel miModelo = null;
        try{
            String titulos []= {"Fuentes | Tipo","Media Palabras"};
            String dts [] = new String[2];
            miModelo = new DefaultTableModel(null, titulos);
            String sql = "select nombre_periodico,tipo_informacion, AVG(palabras) from Titulares where nombre_periodico='"+fuente+"' group by tipo_informacion;";
            PreparedStatement pst = hazConexion().prepareStatement(sql);
            ResultSet rs = pst.executeQuery();
            while(rs.next()){
               String juntar=rs.getString("nombre_periodico")+" | "+rs.getString("tipo_informacion");
               dts[0] = juntar;
               dts[1] = Double.toString(rs.getDouble("AVG(palabras)"));
               miModelo.addRow(dts);
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
        desconectar();
        return miModelo;
    }
    /**
     * Consulta a la base de datos que devuelve el modelo de la tabla con la media de caracteres agrupados por sección.
     * @param fuente
     * @return 
     */
    public DefaultTableModel agruparCaracteres(String fuente){
        DefaultTableModel miModelo = null;
        try{
            String titulos []= {"Fuentes | Tipo","Media Caracteres"};
            String dts [] = new String[2];
            miModelo = new DefaultTableModel(null, titulos);
            String sql = "select nombre_periodico,tipo_informacion, AVG(caracteres) from Titulares where nombre_periodico='"+fuente+"' group by tipo_informacion;";
            PreparedStatement pst = hazConexion().prepareStatement(sql);
            ResultSet rs = pst.executeQuery();
            while(rs.next()){
               String juntar=rs.getString("nombre_periodico")+" | "+rs.getString("tipo_informacion");
               dts[0] = juntar;
               dts[1] = Double.toString(rs.getDouble("AVG(caracteres)"));
               miModelo.addRow(dts);
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
        desconectar();
        return miModelo;
    }
    /**
     * Consulta a la base de datos que devuelve los titulares que contienen una palabra de búsqueda.
     * @return 
     */
    public DefaultTableModel getTitulares_Buscados(){
        DefaultTableModel miModelo = null;
        try{
            String titulos []= {"Titular"};
            String dts [] = new String[3];
            miModelo = new DefaultTableModel(null, titulos);
            String sql = "select titulo,fecha,titulares.nombre_periodico, titulares.tipo_informacion from Titulares, Palabras where palabras.num=titulares.num and busqueda=true;";
            PreparedStatement pst = hazConexion().prepareStatement(sql);
            ResultSet rs = pst.executeQuery();
            while(rs.next()){
               String juntar=rs.getString("titulo");
               dts[0] = rs.getString("titulo");
               dts[0] = juntar;
               miModelo.addRow(dts);
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
        desconectar();
        return miModelo;
    }
    /**
     * Consulta a la base de datos que devuelve los titulares considerados negativos.
     * @return 
     */
    public DefaultTableModel getTitulares_Negativos(){ 
        DefaultTableModel miModelo = null;
        try{
            String titulos []= {"Titular"};
            String dts [] = new String[3];
            miModelo = new DefaultTableModel(null, titulos);
            String sql = "select titulo from Titulares , (select num, sum(negativa) as neg, sum(positiva) as pos from palabras group by num)as myalias where neg>pos and myalias.num=Titulares.num;";
            PreparedStatement pst = hazConexion().prepareStatement(sql);
            ResultSet rs = pst.executeQuery();
            while(rs.next()){
               dts[0] = rs.getString("titulo");
               miModelo.addRow(dts);
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
        desconectar();
        return miModelo;
    }
    /**
     * Consulta a la base de datos que devuelve los titulares considerados positivos.
     * @return 
     */
    public DefaultTableModel getTitulares_Positivos(){
        DefaultTableModel miModelo = null;
        try{
            String titulos []= {"Titular"};
            String dts [] = new String[3];
            miModelo = new DefaultTableModel(null, titulos);
            String sql = "select titulo from Titulares , (select num, sum(negativa) as neg, sum(positiva) as pos from palabras group by num)as myalias where neg<pos and myalias.num=Titulares.num;";
            PreparedStatement pst = hazConexion().prepareStatement(sql);
            ResultSet rs = pst.executeQuery();
            while(rs.next()){
               dts[0] = rs.getString("titulo");
               miModelo.addRow(dts);
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
        desconectar();
        return miModelo;
    }
    /**
     * Actualiza el campo de uso de la tabla de Periódico de la base de datos.
     * @param per 
     */
    public void updateUso(Periodico per) {
        PreparedStatement ps = null;
        String sql = "UPDATE Periodico SET uso=? WHERE nombre_periodico=? AND tipo_informacion=? ";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1, per.isUso() );
            ps.setString(2, per.getNombre_periodico());
            ps.setString(3, per.getTipo_informacion());
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
    }
    /**
     * Establece una palabra de la base de datos como negativa.
     * @param palabra 
     */
    public void setNegativa(String palabra) {
        PreparedStatement ps = null;
        String sql = "UPDATE Palabras SET negativa=? WHERE palabra=? ";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1,true);
            ps.setString(2, palabra);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
    }
    /**
     * Establece una palabra de la base de datos como positiva.
     * @param palabra 
     */
    public void setPositiva(String palabra) {
        PreparedStatement ps = null;
        String sql = "UPDATE Palabras SET positiva=? WHERE palabra=? ";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1,true);
            ps.setString(2, palabra);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
    }
    /**
     * Establece una palabra de la base de datos como palabra de búsqueda.
     * @param palabra 
     */
    public void setBusqueda(String palabra) {
        PreparedStatement ps = null;
        String sql = "UPDATE Palabras SET busqueda=? WHERE palabra=? ";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1,true);
            ps.setString(2,palabra);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
    }
    /**
     * Pone todos los campos de uso de la tabla periódico a false.
     */
    public void resetUso() {
        PreparedStatement ps = null;
        String sql = "UPDATE Periodico SET uso=?";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1, false);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        }  
    }
    /**
     * Pone todos los campos de negativa, de la tabla de Palabras a false.
     */
    public void resetNegativas() {
        PreparedStatement ps = null;
        String sql = "UPDATE Palabras SET negativa=?";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1, false);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        }  
    }
    /**
     * Pone todos los campos de positiva, de la tabla de Palabras a false.
     */
        public void resetPositivas() {
        PreparedStatement ps = null;
        String sql = "UPDATE Palabras SET positiva=?";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1, false);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        }  
    }
    /**
     * Pone todos los campos de busqueda, de la tabla de Palabras a false.
     */    
    public void resetBusqueda() {
        PreparedStatement ps = null;
        String sql = "UPDATE Palabras SET busqueda=?";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setBoolean(1, false);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        }  
    }
    /**
     * Consulta a la base de datos que devuelve las filas de la tabla de Periodico que hay que utilizar para leer.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public ArrayList <String[]> getinfo () throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        ArrayList <String[]> lista = new ArrayList <String[]> ();
        String sql =          " SELECT nombre_periodico, tipo_informacion, link"
                            + " FROM Periodico"
                            + " WHERE uso=true ;";
        
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        while (rs.next()) {
            String dts [] = new String[3];
               dts[0] = rs.getString("nombre_periodico");
               dts[1] = rs.getString("tipo_informacion");
               dts[2] = rs.getString("link");
            lista.add(dts);
        }
        ps.close();
        desconectar();
        return lista;
    }
    /**
     * Incrementa en una unidad el contador de la palabra dada
     * @param palabra 
     */
    public void sumarcontador(Palabras palabra){
        PreparedStatement ps = null;
        String sql = "UPDATE Palabras SET contador=? WHERE palabra= ? AND nombre_periodico=? AND tipo_informacion=? ";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.setInt(1, palabra.getContador()+1);
            ps.setString(2, palabra.getPalabra());
            ps.setString(3, palabra.getNombre_periodico());
            ps.setString(4, palabra.getTipo_informacion());
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        }        
    }
    /**
     * Inserción de una palbra en la base de datos en la tabla de Palabras, con los correspondientes campos.
     * @param pal
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public void createPal (Palabras pal) throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql = "INSERT INTO Palabras (palabra,nombre_periodico,tipo_informacion,contador,num,negativa,positiva,busqueda) VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ps.setString(1,pal.getPalabra());
        ps.setString(2,pal.getNombre_periodico());
        ps.setString(3,pal.getTipo_informacion());
        ps.setInt(4,pal.getContador());
        ps.setInt(5,pal.getNum());
        ps.setBoolean(6,pal.getNegativa());
        ps.setBoolean(7,pal.getPositiva());
        ps.setBoolean(8,pal.getBusqueda());
        ps.execute();
        ps.close();
        desconectar();
    }
    /**
     * Obtiene una palabra guardada, la isntancia.
     * @param palabra
     * @param nombre_periodico
     * @param tipo_informacion
     * @param num
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public Palabras getPalabraguardada(String palabra,String nombre_periodico, String tipo_informacion,int num)throws SQLException, JSchException, InstantiationException, IllegalAccessException{        
        String sql =      " SELECT palabra, nombre_periodico,tipo_informacion,contador,num,negativa,positiva,busqueda"
                          + " FROM Palabras"
                          + " WHERE palabra='"+palabra+"' AND nombre_periodico='"+nombre_periodico+"' AND tipo_informacion ='"+tipo_informacion+"' AND num ='"+num+"';";
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        Palabras word = new Palabras();
        while (rs.next()) {
            word.setPalabra(rs.getString("palabra"));
            word.setNombre_periodico(rs.getString("nombre_periodico"));
            word.setTipo_informacion(rs.getString("tipo_informacion"));
            word.setContador(rs.getInt("contador"));
            word.setNum(rs.getInt("num"));
            word.setNegativa(rs.getBoolean("negativa"));
            word.setPositiva(rs.getBoolean("positiva"));
            word.setBusqueda(rs.getBoolean("busqueda"));
        }
        ps.close();
        desconectar();
        return word;
        }
    /**
     * Consulta a la base de datos para determinar la frecuencia de cada una de las palabras.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public ArrayList <String[]> contarPalabras () throws SQLException, JSchException, InstantiationException, IllegalAccessException {
         ArrayList <String[]> lista = new ArrayList <String[]> ();
        String sql =    " SELECT palabra, suma"
                       +" FROM (select palabra ,sum(contador) as suma "
                       +" from Palabras group by palabra) as myalias"
                       +" where suma>1;";
        
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        while (rs.next()) {
            String dts [] = new String[2];
               dts[0] = rs.getString("palabra");
               dts[1] = rs.getString("suma");
            lista.add(dts);
        }
        ps.close();
        desconectar();
        return lista;
    }
    /**
     * Importa  el archivo CSV con las palabras negativas.
     */
    public void importarcsvNegativas() {
        PreparedStatement ps;
        String sql = "load data infile 'C:/ProgramData/MySQL/MySQL Server 8.0/Uploads/negativas.csv' into table negativas fields terminated by ',' lines terminated by '\\n';";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
    }
    /**
     * Importa  el archivo CSV con las palabras positivas.
     */
    public void importarcsvPositivas() {
        PreparedStatement ps;
        String sql = "load data infile 'C:/ProgramData/MySQL/MySQL Server 8.0/Uploads/positivas.csv' into table positivas fields terminated by ',' lines terminated by '\\n';";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
    }
    /**
     * Importa  el archivo CSV con las palabras de búsqueda.
     */
    public void importarcsvBusqueda() {
        PreparedStatement ps;
        String sql = "load data infile 'C:/ProgramData/MySQL/MySQL Server 8.0/Uploads/busqueda.csv' into table busqueda fields terminated by ',' lines terminated by '\\n';";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        }  
    }
    /**
     * Elimina las palbras de la tabla Negativas.
     */
    public void borrarNegativas() {
        PreparedStatement ps;
        String sql = "delete from negativas;";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
    }
    /**
     * Elimina las palbras de la tabla Positivas.
     */
    public void borrarPositivas() {
        PreparedStatement ps;
        String sql = "delete from positivas;";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        } 
        
    }
    /**
     * Elimina las palbras de la tabla Busqueda.
     */
    public void borrarBusqueda() {
        PreparedStatement ps;
        String sql = "delete from busqueda;";
        try {
            ps = hazConexion().prepareStatement(sql);
            ps.execute();
            desconectar();
        } catch (SQLException e) {
            System.err.println(e);
        }  
    }
    /**
     * Obtiene las palabras almacenadas en la tabla negativas.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public ArrayList <String> getNegativas()throws SQLException, JSchException, InstantiationException, IllegalAccessException{    
        String sql =      " SELECT *"
                          + " FROM negativas;";            
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        ArrayList <String> word = new ArrayList();
        while (rs.next()) {
            word.add(rs.getString("palabra"));
        }
        ps.close();
        desconectar();
        return word;
        }
    /**
     * Obtiene las palabras almacenadas en la tabla positivas.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public ArrayList <String> getPositivas()throws SQLException, JSchException, InstantiationException, IllegalAccessException{
            String sql =      " SELECT *"
                            + " FROM positivas;";
            
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        ArrayList <String> word = new ArrayList();
        while (rs.next()) {
            word.add(rs.getString("palabra"));
        }
        ps.close();
        desconectar();
        return word;
    }
    /**
     * Obtiene las palabras almacenadas en la tabla busqueda.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
    public ArrayList <String> getBusqueda()throws SQLException, JSchException, InstantiationException, IllegalAccessException{
        String sql =      " SELECT *"
                        + " FROM busqueda;";
            
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        ArrayList <String> word = new ArrayList();
        while (rs.next()) {
            word.add(rs.getString("palabra"));
        }
        ps.close();
        desconectar();
        return word;
    }
    /**
     * Devuelve las palabras distintas de la tabla palabras.
     * @return
     * @throws SQLException
     * @throws JSchException
     * @throws InstantiationException
     * @throws IllegalAccessException 
     */
     public ArrayList <String> buscaPalabras () throws SQLException, JSchException, InstantiationException, IllegalAccessException {
        String sql =      " SELECT palabra, suma "
                        + "FROM (select palabra ,sum(contador) as suma "
                        + "from Palabras group by palabra) as myalias;";
            
        PreparedStatement ps = hazConexion().prepareStatement(sql);
        ResultSet rs = ps.executeQuery();
        ArrayList <String> word = new ArrayList();
        while (rs.next()) {
            word.add(rs.getString("palabra"));
        }
        ps.close();
        desconectar();
        return word;
     }
}
