package Principal;

import Controlador.control_Results;
import Modelo.FeedMessage;
import Modelo.Palabras;
import Modelo.UsoBD;
import com.jcraft.jsch.JSchException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Clase que llama al RSSFeedParser para realizar la extracción de titulares.
 * @author Adrián
 */
public class ReadTest {
    /**
     * Método para comenzar a extracción de titulares. Llamado por el controlador
     * de la vista de StartWindow.
     * @param ctrl Controloador de la vista, de resultados durante la extracción.
     * @param mod  Instancia para realizar interacciones con la base de datos.
     */
    public static void ejecu(control_Results ctrl, UsoBD mod){
        //Timer para la ejecución periódica de una porción del código.
        Timer timer= new Timer();   
        TimerTask task = new TimerTask() {
        //Almacena los links, nombres del periódico y tipo de información de cada URL.
        String [] links;
        String [] fuente;
        String [] tipo;
        //Número de iteraciones a realizar, uno por cada URL de lectura.
        int iter;
        //Determina la elelción del usuario de finalización de la extracción de titulares.
        String elección;
    
        /**
         * Método ejecutado periódicamente cada 30 minutos, hasta cumplir condición de finalización.
         */
        public void run(){
            //Impide al usuario interactuar con la vista, porque se va a proceder a actualizar la base de datos.
            ctrl.desactivar();
            //Obtiene la elección del usuario de finalización, guardada en el modelo(Nº de horas o Nº de titulares).
            elección=mod.getElección();
            //Para obtener la fecha actual de ejecución, para compararla con la fecha establecidad de finalización.
            Calendar cal = Calendar.getInstance();
            cal.setTime(new Date());	
            Date tempDate = cal.getTime();
            //Fecha de finalización guardada en el modelo.
            Date fin = mod.getHora_fin();
            //Actualización de la vista Results.
            ctrl.iniciar();
            try {
                //Obtiene la información de la selección del usuario,es decir, las URLs que hay que leer.
                ArrayList <String[]> datos=mod.getinfo();
                //Tantas iteraciones como URLs haya que usar.
                iter=datos.size();
                //Para almacenar en arrays los datos leidos de la tabla Periodico de la BD, a utilizar.
                links= new String[iter];
                fuente= new String[iter];
                tipo= new String[iter];
                for(int i =0;i<iter;i++){
                    fuente[i]=datos.get(i)[0];
                    tipo[i]=datos.get(i)[1];
                    links[i]=datos.get(i)[2];
                    }
                }
            catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
                Logger.getLogger(ReadTest.class.getName()).log(Level.SEVERE, null, ex);
                }
            //Iteraciones, una por cada URL.
            for(int k=0;k<iter;k++){
                //Actualización de la variable para cada iteración.
                String usar=links[k];
                //System.out.println(usar);
                //Llamada a RSSFeedParser con la URL.
                RSSFeedParser parser = new RSSFeedParser(usar);
                //Obtención de los datos leidos.
                Feed feed = parser.readFeed();
                //Si es nulo, es que ha habido problema al conectarse con la URL ys e ignora.
                //Se volverá a intentar en la siguiente ejecución del código.
                if(feed!=null){
                    //Para cada uno de los titulares leidos, se crea una instancia FeedMessage y se actulizan los campos.
                    for (int i =0; i< feed.entries.size();i++){
                        FeedMessage message=feed.entries.get(i);
                        FeedMessage titular =new FeedMessage();
                        titular.setTitle(message.getTitle());
                        titular.setTituloLimpio(message.getTituloLimpio());
                        titular.setFecha(message.getFecha());
                        titular.setPalabras(message.getPalabras());
                        titular.setCaracteres(message.getCaracteres());
                        titular.setNombre_periodico(fuente[k]);
                        titular.setTipo_informacion(tipo[k]);
                        try {
                            titular.setNum(mod.numTitular()+1);
                        } catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
                            Logger.getLogger(ReadTest.class.getName()).log(Level.SEVERE, null, ex);
                        }
                        try {
                            //Para determinar si el titular ya estaba almacenado en la BD.
                            Boolean prueba= mod.buscaTitularespecif(titular.getTitle());
                            //Solo se procede si el titular no se encontraba en la BD, es decir, es un titular nuevo.
                            //Si el titular es repetido, se ignora.
                            if (prueba==true){
                                try {
                                    //Inserción del titular en la BD.
                                    mod.createTit(titular);
                                    //Obtención de las palabras de cada titular.
                                    String[] partes_titulo = titular.getTituloLimpio().split(" ");
                                    //Iteración con cada una de las palabras que forman el titular.
                                    for(int l=0;l<partes_titulo.length;l++){
                                        //Creación de una instancia de palabras.
                                        Palabras palabra =new Palabras();
                                        String aux=partes_titulo[l];
                                        //Siempre y cuando la palabra no sea nula.
                                        if(aux.length()>0){
                                            //Se realiza una comprobación de si la palabra, para ese titular ya estaba añadida. 
                                            if(mod.buscaPalabraespecif(aux,titular.getNombre_periodico(),titular.getTipo_informacion(),titular.getNum())){
                                                //Si no está añadida,se actualzian los campos y se inserta en la base de datos, en la tabla Palabras.
                                                palabra.setPalabra(aux);
                                                palabra.setNombre_periodico(titular.getNombre_periodico());
                                                palabra.setTipo_informacion(titular.getTipo_informacion());
                                                palabra.setContador(1);
                                                palabra.setNum(titular.getNum());
                                                palabra.setNegativa(false);
                                                palabra.setPositiva(false);
                                                palabra.setBusqueda(false);
                                                mod.createPal(palabra);
                                            }
                                            else{
                                                //Si ya estaba añadida, se actualiza el contador, incrementándolo una unidad.
                                                palabra= mod.getPalabraguardada(aux,titular.getNombre_periodico(),titular.getTipo_informacion(),titular.getNum());
                                                mod.sumarcontador(palabra);
                                            }
                                            palabra.setPalabra(aux);
                                            palabra.setNombre_periodico(titular.getNombre_periodico());
                                            palabra.setTipo_informacion(titular.getTipo_informacion());
                                        }
                                        //Se pone la variable a null, para recuperar espacio de memoria.
                                        palabra=null;
                                    }
                                    //Actualización de la vista de Results.
                                    ctrl.iniciar();
                                    titular=null;
                                } 
                                catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
                                    Logger.getLogger(ReadTest.class.getName()).log(Level.SEVERE, null, ex);
                                }                     
                            }       
                        } catch (SQLException | JSchException | InstantiationException | IllegalAccessException ex) {
                            Logger.getLogger(ReadTest.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                }
            }
            //Se permite al usuario interactuar con la interfaz, ya que la actualización de la BD ha concluido.
            ctrl.activar();
            //Comprobación para ver si la extracción de titulares debe concluir.
            //Si la elección fue Nº de horas, y la fecha actual es posterior a la de fin, se acaba el Timer.
            if((elección.equals("Horas") && tempDate.after(fin))){
                //Fin de la ejecución repetida de código.
                cancel();
                //Paso al controlador de Results, para cambiar de vista, a la de Sumamry.
                ctrl.finalizar();
            }
            //Si la elección fue Nº de titulares y ya se han guardado los establecidos, se acaba el Timer.
            if((elección.equals("Titulares") && ctrl.miVista.jProgressBar1.getValue()==100)){
                //Fin de la ejecución repetida de código.
                cancel();
                //Paso al controlador de Results, para cambiar de vista, a la de Sumamry.
                ctrl.finalizar();
            } 
        }
        };timer.schedule(task, 0,900000); //Aquí se especifica en milisegundos, cada cuanto se ejecuta el código.       
    }
}